import { Component, OnInit, Injector } from '@angular/core';
import { Router } from '@angular/router';

import { timer } from 'rxjs';

import { environment } from '@env/environment';
import { AuthenticationService, AuthenticationOAuth2Service } from '@app/core';
import { CreateSubscriptionService } from '@app/shared';

@Component({
  selector: 'app-sign-in-callback',
  templateUrl: './sign-in-callback.component.html',
  styleUrls: ['./sign-in-callback.component.scss']
})
export class SignInCallbackComponent implements OnInit {
  loading: boolean;
  success: boolean;
  loginUrl: string = '/login';


  constructor(
    private router: Router,
    private injector: Injector,
    private authenticationOAuth2Service: AuthenticationOAuth2Service,
    private subscriptionService: CreateSubscriptionService) {
    this.loading = true;
    if (environment.authentication.type === 'oauth2') {
      this.loginUrl = '';
    }
  }

  ngOnInit() {

    timer(10).subscribe(() => {
      this.authenticationOAuth2Service.signinCallback().then(() => {
        this.loading = false;
        this.success = true;
      }).then(() => {
        const claims: any = this.authenticationOAuth2Service.claims;
        const authenticationService: AuthenticationService = this.injector.get(AuthenticationService);
        const currentRouting: string = sessionStorage.getItem('currentRouting');
        if (environment.authentication.useServiceV1 && claims.authToken &&
          (!authenticationService.isAuthenticated() ||
            authenticationService.credentials.token !== claims.authToken)) {
          authenticationService.loginByAuthToken(claims.authToken)
            .subscribe(() => {
              this.navigateUrl(currentRouting);
            });
        } else {
          this.navigateUrl(currentRouting);
        }
      }).catch(() => {
        this.loading = false;
        this.success = false;
      });
    });
  }

  private navigateUrl(currentRouting: string) {
    let promise;
    if (currentRouting !== null && currentRouting !== '/login') {
      if (currentRouting === '/authentication/callback') {
        promise = this.router.navigateByUrl(`/dashboard`);
      } else {
        promise = this.router.navigateByUrl(`/${currentRouting}`);
      }
    } else {
      promise = this.router.navigateByUrl(`/dashboard`);
    }

    promise.then((s) => {
      this.subscriptionService.login$.next(s);
    });
    return promise;
  }
}
