import { NgModule } from '@angular/core';
import { Routes, RouterModule } from '@angular/router';

import { RouteExtensions } from '@app/core/route.service';

const routes: Routes = RouteExtensions.withShell([
  { path: '', loadChildren: '../home/home.module#HomeModule' },
  { path: '', loadChildren: '../about/about.module#AboutModule' },
  // 若果想要懒加载，请不要让path为空
  {
    path: 'contact-us',
    loadChildren: '../contact-us/contact-us.module#ContactUsModule',
    data: { title: '联系我们' }
  },
  {
    path: 'messages',
    loadChildren: '../messages/web-message.module#WebMessageModule',
    data: { title: '我的消息' }
  },
  {
    path: 'help',
    loadChildren: '../help/help.module#HelpModule',
    data: { title: '帮助中心' }
  },
<% if (props.demo) {-%>
  {
    path: 'counter',
    loadChildren: '../counter-demo/counter-demo.module#CounterDemoModule',
    data: { title: '计数器' }
  },
  {
    path: 'roles',
    loadChildren: '../role-management/role-management.module#RoleManagementModule',
    data: { title: '角色列表' }
  },
  {
    path: 'form',
    loadChildren: '../form-demo/form-demo.module#FormDemoModule',
    data: { title: 'form表单' }
  },
  {
    path: 'selected', loadChildren: '../select-selection-demo/select-selection-demo.module#SelectSelectionDemoModule',
    data: { title: 'select选择器' }
  },
  {
    path: 'select2', loadChildren: '../select2-demo/select2-demo.module#Select2DemoModule',
    data: { title: 'select2选择器' }
  },
  {
    path: 'switch', loadChildren: '../switch-demo/switch-demo.module#SwitchDemoModule',
    data: { title: 'switch组件' }
  },
  {
    path: 'datepicker', loadChildren: '../datepicker-demo/datepicker-demo.module#DatepickerDemoModule',
    data: { title: '时间选择器' }
  },
  {
    path: 'validator', loadChildren: '../form-validator-demo/form-validator-demo.module#FormValidatorDemoModule',
    data: { title: '表单验证器' }
  },
  {
    path: 'orders',
    loadChildren: '../order-management/order-management.module#OrderManagementModule',
    data: { title: '订单管理' }
  },
  {
    path: 'upload', loadChildren: '../file-upload-demo/file-upload.module#FileUploadDemoModule',
    data: { title: '文件上传' }
  },
  {
    path: 'upload-oss', loadChildren: '../file-upload-oss-demo/file-upload-oss-demo.module#FileUploadOssDemoModule',
    data: { title: '上传文件到OSS' }
  },
  {
    path: 'vehicle-select',
    loadChildren: '../vehicle-selection-demo/vehicle-management-demo.module#VehicleManagementDemoModule',
    data: { title: '车型选择器' }
  },
  {
    path: 'treeview', loadChildren: '../treeview-demo/treeview-demo.module#TreeviewDemoModule',
    data: { title: '树结构' }
  },
  {
    path: 'tabs', loadChildren: '../tabs-demo/tabs-demo.module#TabsDemoModule',
    data: { title: 'tabs组件' }
  },
  {
    path: 'chart', loadChildren: '../chart-demo/chart-demo.module#ChartDemoModule',
    data: { title: '图表' }
  },
  {
    path: 'button', loadChildren: '../button-demo/button-demo.module#ButtonDemoModule',
    data: { title: '按钮' }
  },
  {
    path: 'collapse', loadChildren: '../collapse-demo/collapse-demo.module#CollapseDemoModule',
    data: { title: 'collapse折叠面板' }
  },
  {
    path: 'slider', loadChildren: '../slider-demo/slider-demo.module#SliderDemoModule',
    data: { title: 'slider滑块' }
  },
  {
    path: 'dropdown',
    loadChildren: '../dropdown-demo/dropdown-demo.module#DropdownDemoModule',
    data: { title: '下拉选择器' }
  },
  {
    path: 'tips',
    loadChildren: '../tips-demo/tips-demo.module#TipsDemoModule',
    data: { title: 'tips提示' }
  },
  {
    path: 'tag',
    loadChildren: '../tag-demo/tag-demo.module#TagDemoModule',
    data: { title: 'tag标签' }
  },
  {
    path: 'badge',
    loadChildren: '../badge-demo/badge-demo.module#BadgeDemoModule',
    data: { title: 'badge标记' }
  },
  {
    path: 'rate',
    loadChildren: '../rate-demo/rate-demo.module#RateDemoModule',
    data: { title: 'rate评分' }
  },
  {
    path: 'progress',
    loadChildren: '../progress-demo/progress-demo.module#ProgressDemoModule',
    data: { title: '进度条' }
  },
  {
    path: 'steps',
    loadChildren: '../steps-demo/steps-demo.module#StepsDemoModule',
    data: { title: 'steps步骤条' }
  },
  {
    path: 'card',
    loadChildren: '../card-demo/card-demo.module#CardDemoModule',
    data: { title: 'card卡片' }
  },
  {
    path: 'no-page-footer',
    loadChildren: '../no-page-footer-demo/no-page-footer-demo.module#NoPageFooterDemoModule',
    data: { title: 'noPageFooter Demo' }
  },
  {
    path: 'trim',
    loadChildren: '../zr-trim-demo/zr-trim-demo.module#ZrTrimDemoModule',
    data: { title: 'zr-trim' }
  },
  {
    path: 'echarts', loadChildren: '../echarts-demo/echarts-demo.module#EchartsDemoModule',
    data: { title: 'echarts图表' }
  },
  // { path: '', loadChildren: '../terms/terms.module#TermsModule' },
  // { path: '', loadChildren: '../privacy/privacy.module#PrivacyModule' },
<% } -%>
]);

@NgModule({
  imports: [RouterModule.forChild(routes)],
  exports: [RouterModule],
  providers: []
})

export class CoreRoutingModule { }
