import { Component, OnInit } from '@angular/core';
import { Router } from '@angular/router';

import { finalize } from 'rxjs/operators';

import { Logger } from '@core/logger.service';
import { LoggerFactory } from '@core/logger-factory.service';

import { OrdersService } from '../../shared/orders.service';
import { Order } from '../../shared/models';

@Component({
  selector: 'app-create-order',
  templateUrl: './create-order.component.html',
  styleUrls: ['./create-order.component.scss']
})
export class CreateOrderComponent implements OnInit {
  log: Logger;
  title = '新建订单';
  saving = false;

  constructor(
    private router: Router,
    private ordersService: OrdersService,
    private loggerFactory: LoggerFactory
  ) {
    this.log = this.loggerFactory.getLogger(`创建订单`);
  }

  ngOnInit() {}

  submit(order: Order) {
    this.saving = true;
    this.ordersService
      .createOrder(order)
      .pipe(finalize(() => (this.saving = false)))
      .subscribe(
        () => {
          this.router.navigate(['/orders']);
          this.log.success('订单创建成功!');
        },
        (error) => this.log.error('订单创建失败。', error)
      );
  }
}
