import { Component, OnInit, Input, AfterViewInit } from '@angular/core';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';

import { finalize } from 'rxjs/operators';
import { BsModalService, BsModalRef } from 'ngx-bootstrap';

import { Logger } from '@core/logger.service';
import { LoggerFactory } from '@core/logger-factory.service';
import { OrdersService } from '../../shared/orders.service';
import { Order, hearFromItems } from '../../shared/models';

declare const lengthStorageArea: any;

@Component({
  selector: 'app-edit-order-modal',
  templateUrl: './edit-order-modal.component.html',
  styleUrls: ['./edit-order-modal.component.scss']
})
export class EditOrderModalComponent implements OnInit, AfterViewInit {
  @Input() data: Order;

  log: Logger;
  form: FormGroup;
  saving = false;

  hearFromItems = hearFromItems;

  constructor(
    public activeModal: BsModalRef,
    private formBuilder: FormBuilder,
    private ordersService: OrdersService,
    private loggerFactory: LoggerFactory,
    public modalService: BsModalService) {
    this.log = this.loggerFactory.getLogger('订单编辑');
    this.buildForm();
  }

  ngOnInit() { }

  ngAfterViewInit(): void {
    this.form.patchValue(this.data);
  }

  submit() {
    this.saving = true;
    this.form.value[`id`] = this.data.id;
    this.ordersService.updateOrder(this.form.value)
      .pipe(finalize(() => this.saving = false))
      .subscribe(() => {
        this.modalService.onHidden.emit(this.form.value);
        this.activeModal.hide();

        this.log.success(`订单编辑成功!`);
      }, error => this.log.error('订单保存失败。', error));
  }

  private buildForm() {
    this.form = this.formBuilder.group({
      subject: [null, [Validators.required, Validators.maxLength(50)]],
      date: [null, [Validators.required,
      Validators.pattern(lengthStorageArea.dateValPattern)]],
      hearFrom: [null, [Validators.required]],
      price: [null, [Validators.required]],
      amount: [null, [Validators.required]]
    });
  }

}
