import { Component, OnInit, EventEmitter } from '@angular/core';
import { Router, ActivatedRoute } from '@angular/router';

import { map, finalize } from 'rxjs/operators';

import { Logger } from '@core/logger.service';
import { LoggerFactory } from '@core/logger-factory.service';
import { OrdersService } from '../../shared/orders.service';
import { Order } from '../../shared/models';

@Component({
  selector: 'app-edit-order',
  templateUrl: './edit-order.component.html',
  styleUrls: ['./edit-order.component.scss']
})
export class EditOrderComponent implements OnInit {
  log: Logger;
  title = '编辑订单';
  saving = false;
  order$ = new EventEmitter<Order>();

  private _id: string;

  constructor(
    private router: Router,
    private route: ActivatedRoute,
    private ordersService: OrdersService,
    private loggerFactory: LoggerFactory
  ) {
    this.log = this.loggerFactory.getLogger(`订单编辑`);
  }

  ngOnInit() {
    this.load();
  }

  submit(order: Order) {
    this.saving = true;
    this.buildFormValue(order);
    this.ordersService
      .updateOrder(order)
      .pipe(finalize(() => (this.saving = false)))
      .subscribe(
        () => {
          this.router.navigate(['/orders']);
          this.log.success(`订单 ${order.subject} 编辑成功!`);
        },
        (error) =>
          this.log.error(`订单 ${order.subject} 保存失败，失败信息:`, error)
      );
  }

  buildFormValue(order: Order): Order {
    order.id = this._id;
    return order;
  }

  private load() {
    this.route.params.pipe(map((params) => params.id)).subscribe((id) => {
      if (id) {
        this.ordersService.getOrder(id).subscribe(
          (order) => {
            this._id = id;
            this.order$.next(order);
          },
          (error) => this.log.error('订单数据获取失败, 失败信息：', error)
        );
      }
    });
  }
}
