import {
  Component,
  OnInit,
  Input,
  EventEmitter,
  Output,
  ChangeDetectionStrategy
} from '@angular/core';
import { FormGroup, FormBuilder, Validators } from '@angular/forms';

import { Logger } from '@app/core';
import { Order, hearFromItems } from '../../shared/models';

declare const lengthStorageArea: any;

@Component({
  selector: 'app-order-ui',
  templateUrl: './order-ui.component.html',
  styleUrls: ['./order-ui.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush
})
export class OrderUiComponent implements OnInit {
  @Input() title: string;
  @Input() saving = false;
  @Input() backLink = '../';
  @Input() get order(): Order {
    return this._order;
  }
  set order(order: Order) {
    if (!this._order === !order) {
      return;
    }

    this._order = order;
    this.form.patchValue(this._order ? this._order : {});
  }

  @Output() save: EventEmitter<Order> = new EventEmitter<Order>();

  log: Logger;
  form: FormGroup;
  hearFromItems = hearFromItems;

  private _order: Order;

  constructor(private formBuilder: FormBuilder) {
    this.buildForm();
  }

  ngOnInit() {}

  submit() {
    if (!this.saving) {
      this.save.next(this.form.value);
    }
  }

  renderDay(date: Date) {
    switch (date.getDate()) {
      case 8:
        return 'text-danger';
      default:
        return null;
    }
  }

  private buildForm() {
    this.form = this.formBuilder.group({
      subject: [null, [Validators.required, Validators.maxLength(50)]],
      date: [
        null,
        [
          Validators.required,
          Validators.pattern(lengthStorageArea.dateValPattern)
        ]
      ],
      hearFrom: [null, [Validators.required]],
      price: [null, [Validators.required, Validators.pattern(/^[0-9]*$/)]],
      amount: [null, [Validators.required, Validators.pattern(/^[0-9]*$/)]]
    });
  }
}
