import { Component, OnInit } from '@angular/core';
import { FormGroup } from '@angular/forms';
import { ActivatedRoute } from '@angular/router';

import { map } from 'rxjs/operators';

import { Logger } from '@core/logger.service';
import { LoggerFactory } from '@core/logger-factory.service';
import { OrdersService } from '../../shared/orders.service';
import { Order } from '@app/order-management/shared/models/order';

@Component({
  selector: 'app-order',
  templateUrl: './order.component.html',
  styleUrls: ['./order.component.scss']
})
export class OrderComponent implements OnInit {
  log: Logger;
  title = '订单详细';
  form: FormGroup;
  order: Order = {} as Order;

  constructor(
    private route: ActivatedRoute,
    private loggerFactory: LoggerFactory,
    private ordersService: OrdersService) {
    this.log = this.loggerFactory.getLogger(`订单详情`);
  }

  ngOnInit() { this.load(); }

  private load() {
    this.route.params.pipe(map(params => params.id))
      .subscribe(id => {
        if (id) {
          this.ordersService.getOrder(id)
            .subscribe(order => {
              this.order = order;
            }, error => this.log.error(`订单数据获取失败,失败信息: ${error}`));
        }
      });
  }
}
