import { Injectable } from '@angular/core';
import { HttpClient } from '@angular/common/http';

import { Observable } from 'rxjs';

import { WebApiResultResponse } from '@core/http/web-api-result-response';
import { Order } from './models';
import { QueryPageList, PagedList } from '@app/shared/models/type';

@Injectable({
  providedIn: 'root',
})
export class OrdersService extends WebApiResultResponse {
  constructor(private http: HttpClient) {
    super();
  }

  getOrders(params: QueryPageList): Observable<PagedList<Order>> {
    const url = 'orders';
    return this.http.get<PagedList<Order>>(url, { params: params as any });
  }

  getOrder(orderId: string): Observable<Order> {
    const url = `orders/${orderId}`;
    return this.http.get<Order>(url);
  }

  createOrder(entity: Order): Observable<any> {
    const url = 'orders';
    return this.http.post(url, entity);
  }

  updateOrder(entity: Order): Observable<any> {
    const url = `orders/${entity.id}`;
    return this.http.put(url, entity);
  }

  deleteOrder(entity: Order): Observable<any> {
    const url = `orders/${entity.id}`;
    return this.http.delete(url);
  }
}
