import { DatatableComponent } from '@swimlane/ngx-datatable';
import { GroupOpType, FieldOpItem, Field } from '@zhongruigroup/ngx-query';

/**
 * 查询模板
 *
 */
export interface QueryTemplate {
  name: string;
  template: FilterGroup;
}

/**
 * 分页数据
 */
export interface PagedList<T> {
  /**
   * 数量
   */
  count: number;

  /**
   * 数据
   */
  data: T[];

  /**
   * 页码
   */
  pageIndex?: number;

  /**
   * 每页数量
   *
   */
  pageSize?: number;
}

/**
 * 查询分页事件
 *
 * @export
 */
export interface QueryPageListEvent {
  datatable?: DatatableComponent;
  page?: QueryPageList;
  query?: any;
}

/**
 * 查询分页
 */
export interface QueryPageList {
  pageSize?: number;
  pageIndex?: number;
  sort?: QueryPagingSort[];
  filters?: FilterTerm[];
  filter?: FilterGroup;
}

/**
 * 排序参数
 */
export interface QueryPagingSort {
  /** 获取或设置排序字段 */
  field?: string | undefined;
  /** 获取或设置排序方式 */
  sort?: string | undefined;
  /** 获取排序字符串 */
  sortOrder?: string | undefined;
}

/**
 * 查询组
 *
 */
export interface FilterGroup {
  op: string | GroupOpType;
  groups?: FilterGroup[];
  rules: QueryRule[];
}

/**
 * 查询规则
 *
 */
export interface QueryRule {
  field: string | Field;
  op: string | FieldOpItem;
  data?: any;
  datas?: any[];
}

/**
 * Term查询
 *
 */
export interface QueryTerm {
  filters: FilterTerm[];
  query?: FilterGroup;
}

/**
 * term 条件
 */
export interface FilterTerm {
  field: string;
  op: string;
  term: any;
}

/**
 * 存储分页数据
 *
 * @export
 */
export interface StoragePaging extends QueryPageList {
  rules: QueryRule[];
  sorts: any[];
}

export interface RowDetailToggleEvent<T> {
  type: 'all' | 'row';
  value: T;
}
