import { Injectable } from '@angular/core';

@Injectable({
    providedIn: 'root',
})
export class WindowEventListenerService {
    events: { key: string; value: (event: Event) => void }[] = [];

    constructor() {}

    /**
     * 添加window事件
     *
     */
    addListener(stateKey: string, name: string, listener: (event: Event) => void) {
        const key = this.getKey(stateKey, name);
        const event = this.events.find((s) => s.key === key);

        if (event) {
            window.removeEventListener(name, event.value);
            event.value = listener;
        } else {
            this.events.push({ key: key, value: listener });
        }

        window.addEventListener(name, listener);
    }

    remove(stateKey: string, name: string) {
        const key = this.getKey(stateKey, name);
        window.removeEventListener(name, this.events[key]);
        this.events = this.events.filter((s) => s.key !== key);
    }

    private getKey(key: string, name: string) {
        return key + '||' + name;
    }
}
