import { QueryRule, FilterTerm, FilterGroup, QueryTerm } from '../models/type';
import { NgxQueryComponent } from '@zhongruigroup/ngx-query';

export function getTermQuery(ngxQuery: NgxQueryComponent): QueryTerm {
    const result: QueryTerm = {
      filters: [],
      query: undefined
    };

    if (ngxQuery) {
      let query = ngxQuery.getQuery();
      if (query === undefined) {
        query = ngxQuery.queryTemplates[0].template;
      }

      result.query = query;

      if (!query || !query.rules || !query.rules.length) {
        return result;
      }

      for (const rule of query.rules) {
        filterRuleData(rule);
        setTermFilter(rule, result);
      }
    }

    return result;
  }

/**
 * 设置termFilter
 */
export function setTermFilter(
  rule: QueryRule,
  result: { filters: FilterTerm[]; query?: FilterGroup }
) {
  if (rule.op === 'bt' && rule.datas && rule.datas.length > 1) {
    if (ruleDataValid(rule.datas[0])) {
      result.filters.push({
        field: rule.field as string,
        op: 'ge',
        term: rule.datas[0]
      });
    }
    if (ruleDataValid(rule.datas[1])) {
      result.filters.push({
        field: rule.field as string,
        op: 'le',
        term: rule.datas[1]
      });
    }
  } else {
    if (ruleDataValid(rule.data)) {
      result.filters.push({
        field: rule.field as string,
        op: rule.op as string,
        term: rule.data
      });
    }
  }
}

/**
 * ruleData是否有效
 */
export function ruleDataValid(data: any) {
  if (data && data !== null && data !== '' && data !== undefined) {
    return true;
  }
  return false;
}

/**
 * 过滤Rule数据 避免数字0 和布尔 false不生效
 *
 */
export function filterRuleData(rule: QueryRule) {
  rule.data = filterData(rule.data);

  if (!rule.datas) {
    return;
  }

  if (rule.datas instanceof Array) {
    for (let i = 0; i < rule.datas.length; i++) {
      let data = rule.datas[i];
      data = filterData(data);
      rule.datas[i] = data;
    }
  }
}

export function filterData(data: any): any {
  if (data === 0) {
    data = '0';
  }
  if (data === false) {
    data = 'false';
  }
  return data;
}
