import { Component, OnInit, ChangeDetectorRef, Input, Output, EventEmitter } from '@angular/core';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';

import { BsModalService } from 'ngx-bootstrap';
import { cloneDeep } from 'lodash';
import { TreeNode } from 'primeng/api';

import { Logger } from '@core/logger.service';
import { LoggerFactory } from '@core/logger-factory.service';

@Component({
  selector: 'app-create-node-modal',
  templateUrl: './create-node-modal.component.html',
  styleUrls: ['./create-node-modal.component.scss']
})
export class CreateNodeModalComponent implements OnInit {
  @Input() node: TreeNode;

  log: Logger;
  form: FormGroup;
  saving = false;

  constructor(
    private loggerFactory: LoggerFactory,
    private formBuilder: FormBuilder,
    public modalService: BsModalService,
    private cdf: ChangeDetectorRef) {
    this.log = this.loggerFactory.getLogger();
    this.buildForm();
  }

  ngOnInit() { }

  submit() {
    const node = cloneDeep(this.node);

    let parentId;
    if (node['level'] === 'sameLevel') {
      if (!node.parent) {
        parentId = '';
      } else {
        parentId = node['parentId'];
      }
    } else if (node['level'] === 'subLevel') {
      parentId = node['id'];
    }

    this.saving = true;
    const params = { parentId: parentId, label: this.form.value.nodeName };
    // 改为本地新建
    this.saving = false;
    this.modalService.onHidden.emit(params);
    this.modalService.hide(1);
  }

  private buildForm() {
    this.form = this.formBuilder.group({
      nodeName: [null, [Validators.required, Validators.maxLength(10)]],
    });
  }

}
