import { Component, OnInit } from '@angular/core';

import { BsModalService } from 'ngx-bootstrap';
import { LoggerFactory } from '@core/logger-factory.service';
import { Logger } from '@core/logger.service';
import { TreeNode } from 'primeng/api';

import { EditNodeModalComponent } from '../edit-node-modal/edit-node-modal.component';
import { CreateNodeModalComponent } from '../create-node-modal/create-node-modal.component';
import { defaultSelectedNode } from '@app/utils/default-selection';
import { TreeviewDemoLocalService } from '@app/treeview-demo/shared/treeview-demo-local.service';
import { findComponentView } from '@angular/core/src/render3/util';

@Component({
  selector: 'app-treeview-demo',
  templateUrl: './treeview-demo.component.html',
  styleUrls: ['./treeview-demo.component.scss']
})

export class TreeviewDemoComponent implements OnInit {
  // tree
  log: Logger;
  filesTree: TreeNode[] = [];
  selectedFile: TreeNode;
  items: any;
  loading: Boolean = false;

  nodes: TreeNode[];
  selectedNodes: TreeNode[] = []; // 选中的节点
  defaultArray: string[] = []; // 默认选中的数据


  constructor(
    private treeviewDemoService: TreeviewDemoLocalService,
    private loggerFactory: LoggerFactory,
    private modalService: BsModalService) {
    this.log = this.loggerFactory.getLogger();
  }


  ngOnInit() {
    this.createTreeNode(); // 默认选中已选择的树结构

    this.getRootList(); // 获取一级节点

    this.items = [
      {
        label: '新增同级节点',
        icon: 'fa fa-link',
        command: (event: any) => this.addNode(this.selectedFile, 'sameLevel')
      },
      {
        label: '新增子级节点',
        icon: 'fa fa-chain-broken',
        command: (event: any) => this.addNode(this.selectedFile, 'subLevel')
      },
      {
        label: '编辑节点',
        icon: 'fa fa-edit',
        command: (event: any) => this.editNode(this.selectedFile)
      },
      {
        label: '删除节点',
        icon: 'fa fa-remove',
        command: (event: any) => this.removeNode(this.selectedFile)
      }
    ];
  }

  // 创建默认选中树结构
  createTreeNode() {
    // 要进行默认选中的树结构
    const defaultNodes = [
      {
        label: '第一层',
        id: '1',
        leaf: false,
        parentid: '',
        children: [
          {
            label: '第一层第一个',
            id: '11',
            leaf: false,
            parentid: '1',
            children: [
              { label: 'Expenses.doc', id: '111', parentid: '11', leaf: true },
              { label: 'Resume.doc', id: '112', parentid: '11', leaf: true }]
          },
          {
            label: '第一层第二个',
            id: '12',
            leaf: false,
            parentid: '1',
            children: [{ label: 'Invoices.txt', id: '121', parentid: '12', leaf: true }]
          }]
      },
      {
        label: '第二层',
        id: '2',
        leaf: false,
        parentid: '',
        children: [
          { label: '第二层第一个', id: '21', parentid: '2', leaf: true },
          { label: '第二层第二个', id: '22', parentid: '2', leaf: true },
          { label: '第二层第三个', id: '23', parentid: '2', leaf: true }]
      },
      {
        label: '第三层',
        id: '3',
        leaf: true,
        parentid: '',
      },
    ];
    this.nodes = defaultNodes;
    this.defaultArray = ['112', '121', '23', '3']; // 默认的数据
    this.selectedNodes = defaultSelectedNode(this.nodes, this.defaultArray, this.selectedNodes); // 选中的方法
  }

  // 从本地化操作树的增删改
  // 获取一级
  getRootList() {
    this.loading = true;
    this.treeviewDemoService.getRootList().subscribe((response: any) => {
      this.loading = false;
      this.filesTree = response;
    }, (error: any) => this.log.error('列表获取失败。', error));
  }


  // 获取子级
  getChildrenList(event: any) {
    if (event.node) {
      this.loading = true;
      this.treeviewDemoService.getChildrenList(event.node.id).subscribe((childnodes: any) => {
        this.loading = false;
        event.node.children = childnodes;
      }, (error: any) => this.log.error('子级获取失败。', error));
    }
  }

  nodeSelect(event: any) {
    // this.selectedFile = event;
    this.log.info(`选择：${event.node.label} `);
  }

  nodeUnselect(event: any) {
    this.log.info(`取消选择：${event.node.label} `);
  }

  // 添加节点
  addNode(row: TreeNode, level: string) {
    const initialState = { node: row };
    this.modalService.show(CreateNodeModalComponent, { initialState });
    const onHidden = this.modalService.onHidden.subscribe((params: any) => {
      if (params !== null) {
        switch (level) {
          case 'sameLevel':
            if (row['parentId'] === '') {
              this.filesTree.push(params);
            } else {
              row.parent.children.push(params);
            }
            break;
          case 'subLevel':
            row.children.push(params);
            row.expanded = true;
        }
      }
      onHidden.unsubscribe();
    });
  }


  // 编辑节点
  editNode(row: any) {
    const initialState = { node: row };
    this.modalService.show(EditNodeModalComponent, { initialState });
    const onHidden = this.modalService.onHidden.subscribe((params: any) => {
      if (params !== null) {
        row.label = params.label;
      }
      onHidden.unsubscribe();
    });
  }

  // 移除节点
  removeNode(selectedFile: any) {
    if (selectedFile.parentId === '') {
      this.filesTree.forEach((item: any, index: any) => {
        if (item.id === selectedFile.id) {
          this.filesTree.splice(index, 1);
        }
      });
    } else {
      selectedFile.parent.children.forEach((item: any, index: any) => {
        if (item.id === selectedFile.id) {
          selectedFile.parent.children.splice(index, 1);
        }
      });
    }
  }

  // 拖动节点
  nodeDrop($event: any) {
    const dropNode = $event.dropNode;
    const dragNode = $event.dragNode;
    this.log.success('[' + dragNode.label + ']' + '已拖动到' + '[' + dropNode.label + ']' + '原来的位置');

  }


}
