import { ActivatedRouteSnapshot, RouteReuseStrategy, DetachedRouteHandle } from '@angular/router';
import { compact } from 'lodash';

export class CustomReuseStrategy implements RouteReuseStrategy {
  handlers: { [key: string]: DetachedRouteHandle } = {};
  // 需要存储的component,放到下面的这个数组里面.................
  routesToCache: string[] = ['purchaseOrder', 'CustInfo', 'ProviderInfo', 'providerOrdersList'];

  shouldDetach(route: ActivatedRouteSnapshot): boolean {
    // console.log(this.routesToCache);
    // console.log(this.getKey(route));

    if (!route.routeConfig || route.routeConfig.loadChildren) {
      return false;
    }

    return this.routesToCache.indexOf(this.getKey(route)) > -1;
    // return (route.routeConfig && route.routeConfig.data as IRouteConfigData) != undefined;
  }

  store(route: ActivatedRouteSnapshot, handle: DetachedRouteHandle): void {
    this.handlers[this.getKey(route)] = handle;
  }

  shouldAttach(route: ActivatedRouteSnapshot): boolean {
    return !!route.routeConfig && !!this.handlers[this.getKey(route)];
  }

  retrieve(route: ActivatedRouteSnapshot): DetachedRouteHandle {
    if (!route.routeConfig) {
      return null;
    }

    if (route.routeConfig.loadChildren) return null;
    return this.handlers[this.getKey(route)];
  }

  shouldReuseRoute(future: ActivatedRouteSnapshot, curr: ActivatedRouteSnapshot): boolean {
    return curr.routeConfig === future.routeConfig && JSON.stringify(future.params) === JSON.stringify(curr.params);
  }

  private getKey(route: ActivatedRouteSnapshot) {
    // console.log(route);
    // let key: string;
    // key = (route as any)._routerState.url.replace(/\//g,'_');
    // return key;

    let next = route;
    // Since navigation is usually relative
    // we go down to find out the child to be shown.
    while (next.firstChild) {
      next = next.firstChild;
    }
    const segments = [];
    // Then build a unique key-path by going to the root.
    while (next) {
      segments.push(next.url.join('/'));
      next = next.parent;
    }
    const key = compact(segments.reverse()).join('/');
    return key;
  }
}
